/*
 * libremix -- An audio mixing and sequencing library.
 *
 * Copyright (C) 2001 Commonwealth Scientific and Industrial Research
 * Organisation (CSIRO), Australia.
 * Copyright (C) 2003 Conrad Parker
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * remix.h -- Base types and public interfaces to libremix
 *
 * Conrad Parker <conrad@metadecks.org>, August 2001
 */

#ifndef __REMIX_H__
#define __REMIX_H__

#include <limits.h>

#include "ctxdata.h"

#define RemixNone NULL

typedef int RemixError;

typedef float RemixPCM;

typedef long RemixCount;
#define REMIX_COUNT_MAX (LONG_MAX - 1L)
#define REMIX_COUNT_MIN LONG_MIN
#define REMIX_COUNT_INFINITE REMIX_COUNT_MAX

typedef void RemixOpaque;

#if defined (__REMIX__)
typedef struct _RemixBase RemixBase;
#else
typedef RemixOpaque RemixBase;
#endif

typedef CDScalar RemixParameter;

typedef int RemixFlags;
typedef double RemixSamplerate;
typedef double RemixTempo;
typedef union _RemixTime RemixTime;


/* Errors */
#define REMIX_ERROR_OK              0
#define REMIX_ERROR_INVALID         1
#define REMIX_ERROR_NOENTITY        2
#define REMIX_ERROR_EXISTS          3
#define REMIX_ERROR_SILENCE         4
#define REMIX_ERROR_NOOP            5
#define REMIX_ERROR_SYSTEM          6

typedef enum {
  REMIX_CHANNEL_LEFT,
  REMIX_CHANNEL_RIGHT,
  REMIX_CHANNEL_CENTRE,
  REMIX_CHANNEL_REAR,
  REMIX_CHANNEL_REAR_LEFT,
  REMIX_CHANNEL_REAR_RIGHT,
  REMIX_CHANNEL_REAR_CENTRE,
  REMIX_CHANNEL_LFE /* Low Frequency Effects */,
} RemixChannelName;

typedef enum {
  REMIX_TIME_INVALID,
  REMIX_TIME_SAMPLES,
  REMIX_TIME_SECONDS,
  REMIX_TIME_BEAT24S
} RemixTimeType;

/* Envelope types */
typedef enum {
  REMIX_ENVELOPE_LINEAR,
  REMIX_ENVELOPE_SPLINE
} RemixEnvelopeType;

union _RemixTime {
  long TIME;
  RemixCount samples;
  float seconds;
  int beat24s;
};

typedef enum {
  REMIX_TYPE_BOOL = 0,
  REMIX_TYPE_INT,
  REMIX_TYPE_FLOAT,
  REMIX_TYPE_STRING,
  REMIX_TYPE_BASE,
} RemixParameterType;

typedef enum {
  REMIX_CONSTRAINT_TYPE_NONE = 0,
  REMIX_CONSTRAINT_TYPE_LIST,
  REMIX_CONSTRAINT_TYPE_RANGE,
} RemixConstraintType;


#define REMIX_RANGE_LOWER_BOUND_VALID (1<<0)
#define REMIX_RANGE_UPPER_BOUND_VALID (1<<1)
#define REMIX_RANGE_STEP_VALID        (1<<2)
#define REMIX_RANGE_ALL_VALID (REMIX_RANGE_LOWER_BOUND_VALID | \
                              REMIX_RANGE_UPPER_BOUND_VALID | \
                              REMIX_RANGE_STEP_VALID)

#define REMIX_HINT_DEFAULT  (0)
#define REMIX_HINT_LOG      (1<<0)
#define REMIX_HINT_TIME     (1<<1)
#define REMIX_HINT_FILENAME (1<<2)

#define REMIX_CONSTRAINT_EMPTY ((RemixConstraint){NULL})

#define REMIX_SAMPLES(x) ((RemixTime){(RemixCount)(x)})
#define REMIX_SECONDS(x) ((RemixTime){(float)(x)})
#define REMIX_BEAT24S(x) ((RemixTime){(int)(x)})


#if (defined(__REMIX__) || defined(__REMIX_PLUGIN__))
#include <remix_plugin.h>
#else
typedef RemixOpaque RemixEnv;
typedef RemixOpaque RemixPoint;
typedef RemixOpaque RemixEnvelope;
typedef RemixOpaque RemixChunk;
typedef RemixOpaque RemixChannel;
typedef RemixOpaque RemixStream;
typedef RemixOpaque RemixDeck;
typedef RemixOpaque RemixTrack;
typedef RemixOpaque RemixLayer;
typedef RemixOpaque RemixSound;
typedef RemixOpaque RemixMetaAuthor;
typedef RemixOpaque RemixMetaText;
typedef RemixOpaque RemixPlugin;
typedef RemixOpaque RemixSquareTone;
typedef RemixOpaque RemixMonitor;
#endif


#if defined(__cplusplus)
extern "C" {
#endif

extern CDSet * REMIX_MONO;
extern CDSet * REMIX_STEREO;


/* RemixEnv */
RemixEnv * remix_init (void);
RemixEnv * remix_init_clone (RemixEnv * env);
void remix_purge (RemixEnv * env);
RemixError remix_last_error (RemixEnv * env);
char * remix_error_string (RemixEnv * env, RemixError error);
RemixSamplerate remix_set_samplerate (RemixEnv * env, RemixSamplerate samplerate);
RemixSamplerate remix_get_samplerate (RemixEnv * env);
RemixTempo remix_set_tempo (RemixEnv * env, RemixTempo tempo);
RemixTempo remix_get_tempo (RemixEnv * env);
CDSet * remix_set_channels (RemixEnv * env, CDSet * channelset);
CDSet * remix_get_channels (RemixEnv * env);

#if 0
  /* XXX */
/* Sources: Plugins, Samples etc. */
CDList * remix_open (SOUNDRENDER env, char * filename);
void remix_close (SOUNDRENDER env, RemixPlugin plugin);
#endif

/* Base objects */

RemixPlugin * remix_find_plugin (RemixEnv * env, char * identifier);

RemixBase * remix_new (RemixEnv * env, RemixPlugin * plugin, CDSet * parameters);
CDSet * remix_suggest (RemixEnv * env, RemixPlugin * plugin, CDSet * parameters);

int remix_get_init_parameter_key (RemixEnv * env, RemixPlugin * plugin, char * name);
int remix_get_parameter_key (RemixEnv * env, RemixBase * base, char * name);

RemixParameter remix_set_parameter (RemixEnv * env, RemixBase * base, int key,
				  RemixParameter parameter);
RemixParameter remix_get_parameter (RemixEnv * env, RemixBase * base, int key);
RemixParameterType remix_get_parameter_type (RemixEnv * env, RemixBase * base, int key);


RemixBase * remix_clone_subclass (RemixEnv * env, RemixBase * base);
int remix_destroy (RemixEnv * env, RemixBase * base);
int remix_destroy_list (RemixEnv * env, CDList * list);
RemixBase * remix_prepare (RemixEnv * env, RemixBase * base);
RemixCount remix_process (RemixEnv * env, RemixBase * base, RemixCount count,
		    RemixStream * input, RemixStream * output);
RemixCount remix_length (RemixEnv * env, RemixBase * base);
RemixCount remix_seek (RemixEnv * env, RemixBase * base, RemixCount offset, int whence);
RemixCount remix_tell (RemixEnv * env, RemixBase * base);
int remix_flush (RemixEnv * env, RemixBase * base);


/* Decks */
RemixDeck * remix_deck_new (RemixEnv * env);

RemixCount remix_deck_set_mixlength (RemixEnv * env, RemixDeck * deck, RemixCount mixlength);
RemixCount remix_deck_get_mixlength (RemixEnv * env, RemixDeck * deck);

CDList * remix_deck_get_tracks (RemixEnv * env, RemixDeck * deck);

/* Tracks */
RemixTrack * remix_add_track (RemixEnv * env, RemixDeck * deck);
RemixPCM remix_track_set_gain (RemixEnv * env, RemixTrack * track, RemixPCM gain);
RemixPCM remix_track_get_gain (RemixEnv * env, RemixTrack * track);
RemixCount remix_track_set_mixlength (RemixEnv * env, RemixTrack * track,
				RemixCount mixlength);
RemixCount remix_track_get_mixlength (RemixEnv * env, RemixTrack * track);
void remix_remove_track (RemixEnv * env, RemixTrack * track);

RemixDeck * remix_track_get_deck (RemixEnv * env, RemixTrack * track);


/* Layers */
RemixLayer * remix_add_layer_ontop (RemixEnv * env, RemixTrack * track,
			      RemixTimeType timetype);
RemixLayer * remix_add_layer_above (RemixEnv * env, RemixLayer * above,
			      RemixTimeType timetype);
RemixLayer * remix_move_layer_ontop (RemixEnv * env, RemixLayer * layer, RemixTrack * track);
RemixLayer * remix_move_layer_above (RemixEnv * env, RemixLayer * layer, RemixLayer * above);
RemixLayer * remix_raise_layer (RemixEnv * env, RemixLayer * layer);
RemixLayer * remix_lower_layer (RemixEnv * env, RemixLayer * layer);
RemixTrack * remix_layer_get_track (RemixEnv * env, RemixLayer * layer);
RemixDeck * remix_layer_get_deck (RemixEnv * env, RemixLayer * layer);
RemixTimeType remix_layer_set_timetype (RemixEnv * env, RemixLayer * layer,
				  RemixTimeType new_type);
RemixTimeType remix_layer_get_timetype (RemixEnv * env, RemixLayer * layer);
RemixSound * remix_layer_get_sound_before (RemixEnv * env, RemixLayer * layer,
				     RemixTime time);
RemixSound * remix_layer_get_sound_at (RemixEnv * env, RemixLayer * layer, RemixTime time);
RemixSound * remix_layer_get_sound_after (RemixEnv * env, RemixLayer * layer, RemixTime time);
RemixLayer * remix_get_layer_below (RemixEnv * env, RemixLayer * layer);
RemixLayer * remix_get_layer_above (RemixEnv * env, RemixLayer * layer);

/* Sounds */
RemixSound * remix_add_sound (RemixEnv * env, RemixBase * source,
			      RemixLayer * layer,
			      RemixTime start_time, RemixTime duration);
RemixSound * remix_remove_sound (RemixEnv * env, RemixSound * sound);

RemixBase * remix_sound_set_source (RemixEnv * env, RemixSound * sound,
				    RemixBase * source);
RemixBase * remix_sound_get_source (RemixEnv * env, RemixSound * sound);
RemixLayer * remix_sound_get_layer (RemixEnv * env, RemixSound * sound);
RemixTrack * remix_sound_get_track (RemixEnv * env, RemixSound * sound);
RemixDeck * remix_sound_get_deck (RemixEnv * env, RemixSound * sound);

RemixTime remix_sound_move (RemixEnv * env, RemixSound * sound,
			    RemixTime time);
RemixSound * remix_sound_get_prev (RemixEnv * env, RemixSound * sound);
RemixSound * remix_sound_get_next (RemixEnv * env, RemixSound * sound);

RemixTime remix_sound_set_start_time (RemixEnv * env, RemixSound * sound,
				      RemixTime time);
RemixTime remix_sound_get_start_time (RemixEnv * env, RemixSound * sound);
RemixTime remix_sound_set_duration (RemixEnv * env, RemixSound * sound,
				    RemixTime time);
RemixTime remix_sound_get_duration (RemixEnv * env, RemixSound * sound);

RemixBase * remix_sound_set_rate_envelope (RemixEnv * env, RemixSound * sound,
					   RemixBase * rate_envelope);
RemixBase * remix_sound_get_rate_envelope (RemixEnv * env, RemixSound * sound);
RemixBase * remix_sound_set_gain_envelope (RemixEnv * env, RemixSound * sound,
					   RemixBase * gain_envelope);
RemixBase * remix_sound_get_gain_envelope (RemixEnv * env, RemixSound * sound);
RemixBase * remix_sound_set_blend_envelope (RemixEnv * env, RemixSound * sound,
					    RemixBase * blend_envelope);
RemixBase * remix_sound_get_blend_envelope (RemixEnv * env, RemixSound * sound);


/* Envelopes */
RemixEnvelope * remix_envelope_new (RemixEnv * env, RemixEnvelopeType type);
RemixEnvelopeType remix_envelope_set_type (RemixEnv * env,
					   RemixEnvelope * envelope,
					   RemixEnvelopeType type);
RemixEnvelopeType remix_envelope_get_type (RemixEnv * env, RemixEnvelope * envelope);
RemixTimeType remix_envelope_set_timetype (RemixEnv * env,
					   RemixEnvelope * envelope,
					   RemixTimeType timetype);
RemixTimeType remix_envelope_get_timetype (RemixEnv * env,
					   RemixEnvelope * envelope);
RemixPCM remix_envelope_get_value (RemixEnv * env, RemixEnvelope * envelope,
				   RemixTime time);
RemixTime remix_envelope_get_duration (RemixEnv * env,
				       RemixEnvelope * envelope);
RemixPCM remix_envelope_get_integral (RemixEnv * env, RemixEnvelope * envelope,
				      RemixTime t1, RemixTime t2);
RemixPoint * remix_envelope_add_point (RemixEnv * env,
				       RemixEnvelope * envelope,
				       RemixTime time, RemixPCM value);
RemixEnvelope * remix_envelope_remove_point (RemixEnv * env,
					     RemixEnvelope * envelope,
					     RemixPoint * point);
RemixEnvelope * remix_envelope_scale (RemixEnv * env, RemixEnvelope * envelope,
				      RemixPCM gain);
RemixEnvelope * remix_envelope_shift (RemixEnv * env, RemixEnvelope * envelope,
				      RemixTime delta);


/* Streams */
RemixStream * remix_stream_new (RemixEnv * env);
RemixStream * remix_stream_new_contiguous (RemixEnv * env, RemixCount length);
RemixStream * remix_stream_new_from_buffers (RemixEnv * env, RemixCount length,
					     RemixPCM ** buffers);
RemixCount remix_stream_nr_channels (RemixEnv * env, RemixStream * stream);
RemixChannel * remix_stream_find_channel (RemixEnv * env,
					  RemixStream * stream, int name);
RemixChannel * remix_stream_add_channel (RemixEnv * env,
					 RemixStream * stream, int name);
RemixStream * remix_stream_remove_channel (RemixEnv * env,
					   RemixStream * stream, int name);
RemixStream * remix_stream_add_chunks (RemixEnv * env, RemixStream * stream,
				       RemixCount offset, RemixCount length);

RemixCount remix_stream_write0 (RemixEnv * env, RemixStream * stream,
				RemixCount count);
RemixCount remix_stream_write (RemixEnv * env, RemixStream * stream,
			       RemixCount count, RemixStream * data);
RemixCount remix_stream_copy (RemixEnv * env, RemixStream * src,
			      RemixStream * dest, RemixCount count);
RemixCount remix_stream_gain (RemixEnv * env, RemixStream * stream,
			      RemixCount count, RemixPCM gain);
RemixCount remix_stream_mix (RemixEnv * env, RemixStream * src,
			     RemixStream * dest, RemixCount count);
RemixCount remix_stream_mult (RemixEnv * env, RemixStream * src,
			      RemixStream * dest, RemixCount count);
RemixCount remix_streams_mix (RemixEnv * env, CDList * streams,
			      RemixStream * dest, RemixCount count);
RemixCount remix_stream_fade (RemixEnv * env, RemixStream * src,
			      RemixStream * dest, RemixCount count);
RemixCount remix_stream_blend (RemixEnv * env, RemixStream * src,
			       RemixStream * dest,
			       RemixStream * blend, RemixCount count);

RemixCount remix_stream_interleave_2 (RemixEnv * env, RemixStream * stream,
				      int name1, int name2,
				      RemixPCM * dest, RemixCount count);
RemixCount remix_stream_deinterleave_2 (RemixEnv * env, RemixStream * stream,
					int name1, int name2,
					RemixPCM * src, RemixCount count);

/* Channels */
RemixChannel * remix_add_channel (RemixEnv * env, RemixStream * stream,
				  RemixChannelName name);
void remix_free_channel (RemixEnv * env, RemixChannel * channel);
void remix_remove_chunk (RemixEnv * env, RemixChunk * chunk);

/* Chunks */
RemixChunk * remix_add_chunk (RemixEnv * env, RemixChannel * channel,
			      RemixCount length, RemixCount offset);
int remix_chunk_later (RemixEnv * env, RemixChunk * u1, RemixChunk * u2);
RemixCount remix_chunk_clear (RemixEnv * env, RemixChunk * chunk);

/* SquareTone */
RemixSquareTone * remix_squaretone_new (RemixEnv * env, float frequency);
float remix_squaretone_set_frequency (RemixEnv * env,
				      RemixSquareTone * squaretone,
				      float frequency);
float remix_squaretone_get_frequency (RemixEnv * env,
				      RemixSquareTone * squaretone);

/* Monitor */
RemixMonitor * remix_monitor_new (RemixEnv * env);


/* Time */
RemixTime remix_time_convert (RemixEnv * env, RemixTime time,
			      RemixTimeType old_type, RemixTimeType new_type);
RemixTime remix_time_zero (RemixTimeType type);
RemixTime remix_time_invalid (RemixTimeType type);
int remix_time_is_invalid (RemixTimeType type, RemixTime time);
RemixTime remix_time_add (RemixTimeType type, RemixTime t1, RemixTime t2);
RemixTime remix_time_sub (RemixTimeType type, RemixTime t1, RemixTime t2);
RemixTime remix_time_min (RemixTimeType type, RemixTime t1, RemixTime t2);
RemixTime remix_time_max (RemixTimeType type, RemixTime t1, RemixTime t2);
int remix_time_eq (RemixTimeType type, RemixTime t1, RemixTime t2);
int remix_time_gt (RemixTimeType type, RemixTime t1, RemixTime t2);
int remix_time_lt (RemixTimeType type, RemixTime t1, RemixTime t2);
int remix_time_ge (RemixTimeType type, RemixTime t1, RemixTime t2);
int remix_time_le (RemixTimeType type, RemixTime t1, RemixTime t2);


RemixCount remix_set_mixlength (RemixEnv * env, RemixCount mixlength);
RemixCount remix_get_mixlength (RemixEnv * env);


int remix_is_writeable (RemixEnv * env, RemixBase * base);
int remix_is_seekable (RemixEnv * env, RemixBase * base);
int remix_is_cacheable (RemixEnv * env, RemixBase * base);
int remix_is_causal (RemixEnv * env, RemixBase * base);

char * remix_set_name (RemixEnv * env, RemixBase * base, char * name);
char * remix_get_name (RemixEnv * env, RemixBase * base);

  /* XXX: Move meta out of remix.h ??? */

char * remix_meta_text_get_identifier (RemixEnv * env, RemixMetaText * mt);
char * remix_meta_text_set_identifier (RemixEnv * env, RemixMetaText * mt,
				       char * identifier);
char * remix_meta_text_get_category (RemixEnv * env, RemixMetaText * mt);
char * remix_meta_text_set_category (RemixEnv * env, RemixMetaText * mt,
				     char * category);
char * remix_meta_text_get_description (RemixEnv * env, RemixMetaText * mt);
char * remix_meta_text_set_description (RemixEnv * env, RemixMetaText * mt,
					char * description);
char * remix_meta_text_get_copyright (RemixEnv * env, RemixMetaText * mt);
char * remix_meta_text_set_copyright (RemixEnv * env, RemixMetaText * mt,
				      char * copyright);
char * remix_meta_text_get_url (RemixEnv * env, RemixMetaText * mt);
char * remix_meta_text_set_url (RemixEnv * env, RemixMetaText * mt,
				char * url);
CDList * remix_meta_text_get_authors (RemixEnv * env, RemixMetaText * mt);
void remix_meta_text_add_author (RemixEnv * env, RemixMetaText * mt,
				 char * name, char * email);

#if defined(__cplusplus)
}
#endif

#endif /* __REMIX_H__ */
